clear
set more off

local path    = "U:\tf\Matteo\Rshock\Replication"	// where files for regressions are


cls
cd `path'

adopath ++ `path'\ADOFILES
set scrollbufsize 300000

use datasetforplots, clear

set scheme s1color

label var auxvvf7     "Dollar Peg"
label var auxvvf8     "Trade with U.S."
label var auxvvfindex "Vulnerability Index"

replace auxvvf7 = auxvvf7 + 0.008 if countrylong=="Mexico"
replace auxvvf7 = auxvvf7 - 0.008 if countrylong=="Korea"
replace auxvvf7 = auxvvf7 + 0.008 if countrylong=="Canada"
replace auxvvf7 = auxvvf7 - 0.008 if countrylong=="United Kingdom"

// SAVE DATA
preserve 

replace countrylong = "UK" if countrylong == "United Kingdom"
local qin "1965q1"
local qfn "2016q4"
keep if tin( `qin' ,`qfn' ) & (countrylong == "China" | countrylong == "Korea"| countrylong == "Mexico" | countrylong == "Canada" | countrylong == "Japan"| countrylong == "UK")

keep auxvvf7 auxvvf8 auxvvfindex quarter countrylong

gen yy = yofd(dofq(quarter)) 
gen qq = quarter(dofq(quarter)) 
gen yq = yy + (qq-1)/4

drop yy qq 

reshape wide auxvvf7 auxvvf8 auxvvfindex, i(yq) j(countrylong) string

cd `out_dir'
outsheet using FRAG_VARS_file.csv, comma noquote replace
cd `path'

restore



// Plots

xtline auxvvf8 if (countrylong == "China" | countrylong == "Korea"| countrylong == "Mexico") & tin(1965q1,2016q4), ///
overlay title(`: variable label auxvvf8') ytitle("") xtitle("") ylabel(0(.2)1) xlab(20(40)220) legend(rows(1)) tlabel(, format(%tqCY)) name(TRADE_EME, replace) ///
plot1opts(lwidth(medthick)) plot2opts(lwidth(medthick)) plot3opts(lwidth(medthick))

xtline auxvvf8 if (countrylong == "Canada" | countrylong == "Japan"| countrylong == "United Kingdom") & tin(1965q1,2016q4), ///
overlay title(`: variable label auxvvf8') ytitle("") xtitle("") ylabel(0(.2)1) xlab(20(40)220) legend(rows(1)) tlabel(, format(%tqCY)) name(TRADE_AFE, replace) ///
plot1opts(lwidth(medthick)) plot2opts(lwidth(medthick)) plot3opts(lwidth(medthick))


*FIX EQUAL SERIES
replace auxvvf7 = auxvvf7 + 0.008 if countrylong=="Mexico"
replace auxvvf7 = auxvvf7 - 0.008 if countrylong=="Korea"
replace auxvvf7 = auxvvf7 + 0.008 if countrylong=="Canada"
replace auxvvf7 = auxvvf7 - 0.008 if countrylong=="United Kingdom"

xtline auxvvf7 if (countrylong == "China" | countrylong == "Korea"| countrylong == "Mexico") & tin(1965q1,2016q4), ///
overlay title(`: variable label auxvvf7') ytitle("") xtitle("") ylabel(0(.2)1) xlab(20(40)220) tlabel(, format(%tqCY)) name(XR_EME, replace) ///
plot1opts(lwidth(medthick)) plot2opts(lwidth(medthick)) plot3opts(lwidth(medthick))


xtline auxvvf7 if (countrylong == "Canada" | countrylong == "Japan"| countrylong == "United Kingdom") & tin(1965q1,2016q4), ///
overlay title(`: variable label auxvvf7') ytitle("") xtitle("") ylabel(0(.2)1) xlab(20(40)220) tlabel(, format(%tqCY)) name(XR_AFE, replace) ///
plot1opts(lwidth(medthick)) plot2opts(lwidth(medthick)) plot3opts(lwidth(medthick))


xtline auxvvfindex if (countrylong == "China" | countrylong == "Korea"| countrylong == "Mexico") & tin(1965q1,2016q4), ///
overlay title(`: variable label auxvvfindex') ytitle("") xtitle("") ylabel(0(.2)1) xlab(20(40)220) tlabel(, format(%tqCY)) name(F_EME, replace) ///
plot1opts(lwidth(medthick)) plot2opts(lwidth(medthick)) plot3opts(lwidth(medthick))


xtline auxvvfindex if (countrylong == "Canada" | countrylong == "Japan"| countrylong == "United Kingdom") & tin(1965q1,2016q4), ///
overlay title(`: variable label auxvvfindex') ytitle("") xtitle("") ylabel(0(.2)1) xlab(20(40)220) tlabel(, format(%tqCY)) name(F_AFE, replace) ///
plot1opts(lwidth(medthick)) plot2opts(lwidth(medthick)) plot3opts(lwidth(medthick))


graph combine XR_AFE TRADE_AFE F_AFE , rows(1) name(AFE, replace) 
graph combine XR_EME TRADE_EME F_EME , rows(1) name(EME, replace)
graph combine AFE EME, rows(2) name(AFEEME, replace)

gr export FIGURE5.pdf, mag(100) replace name(AFEEME) 

// Save data for LaTex

local qin "1965q1"
local qfn "2016q4"


